"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.LANGS = void 0;
exports.getHighlighter = getHighlighter;
exports.getSafeLanguage = getSafeLanguage;
exports.isValidLanguage = isValidLanguage;
var _core = require("@shikijs/core");
var _css = _interopRequireDefault(require("shiki/dist/langs/css.mjs"));
var _html = _interopRequireDefault(require("shiki/dist/langs/html.mjs"));
var _javascript = _interopRequireDefault(require("shiki/dist/langs/javascript.mjs"));
var _json = _interopRequireDefault(require("shiki/dist/langs/json.mjs"));
var _markdown = _interopRequireDefault(require("shiki/dist/langs/markdown.mjs"));
var _php = _interopRequireDefault(require("shiki/dist/langs/php.mjs"));
var _tsx = _interopRequireDefault(require("shiki/dist/langs/tsx.mjs"));
var _typescript = _interopRequireDefault(require("shiki/dist/langs/typescript.mjs"));
var _githubDark = _interopRequireDefault(require("shiki/dist/themes/github-dark.mjs"));
function _interopRequireDefault(e) { return e && e.__esModule ? e : { default: e }; }
/* eslint-disable no-console */

const DEBUG = true;
function log(message, ...args) {
  if (DEBUG) {
    console.log(`[ShikiContext] ${message}`, ...args);
  }
}
class CustomRegexScanner {
  constructor(patterns) {
    this.regexes = patterns.map(pattern => {
      try {
        return new RegExp(pattern, 'g');
      } catch (e) {
        log('Error creating regex for pattern:', pattern, e);
        return /./g; // Fallback to match any character
      }
    });
  }
  findNextMatchSync(string, startPosition) {
    const stringContent = string.toString();
    for (let i = 0; i < this.regexes.length; i++) {
      const regex = this.regexes[i];
      if (!regex) continue;
      regex.lastIndex = startPosition;
      const match = regex.exec(stringContent);
      if (match) {
        return {
          index: match.index,
          captureIndices: [{
            start: match.index,
            end: match.index + match[0].length,
            length: match[0].length
          }],
          scanner: this
        };
      }
    }
    return null;
  }
  dispose() {
    this.regexes = [];
  }
}
class CustomRegexString {
  constructor(content) {
    this.content = content;
  }
  toString() {
    return this.content;
  }
}
function createCustomRegexEngine() {
  const engine = {
    createScanner(patterns) {
      return new CustomRegexScanner(patterns);
    },
    createString(content) {
      return new CustomRegexString(content);
    }
  };
  return engine;
}

// Process the language definitions
function processLanguage(lang) {
  if (!lang) {
    log('Warning: Null language definition');
    return null;
  }

  // Extract or create necessary properties
  const processed = {
    ...lang,
    id: lang.id || lang.scopeName?.split('.').pop() || 'unknown',
    scopeName: lang.scopeName || `source.${lang.id || 'unknown'}`,
    grammar: lang.grammar || {},
    embeddedLanguages: lang.embeddedLanguages || {}
  };
  log(`Processing language ${processed.id}:`, {
    scopeName: processed.scopeName,
    hasGrammar: Boolean(processed.grammar),
    embeddedLangs: Object.keys(processed.embeddedLanguages)
  });
  return processed;
}

// Map of all available languages with processed implementations
const LANGS = exports.LANGS = {
  typescript: processLanguage(_typescript.default),
  javascript: processLanguage(_javascript.default),
  json: processLanguage(_json.default),
  html: processLanguage(_html.default),
  css: processLanguage(_css.default),
  markdown: processLanguage(_markdown.default),
  tsx: processLanguage(_tsx.default),
  php: processLanguage(_php.default),
  plaintext: {
    id: 'plaintext',
    scopeName: 'source.plaintext',
    grammar: {
      patterns: [{
        match: '.*',
        name: 'text.plain'
      }]
    }
  }
};
let highlighterInstance = null;
async function getHighlighter() {
  if (highlighterInstance) {
    log('Returning cached highlighter instance');
    return highlighterInstance;
  }
  try {
    log('Creating new highlighter instance...');

    // Initialize custom regex engine
    log('Initializing custom regex engine...');
    const engine = createCustomRegexEngine();
    log('Engine created with capabilities:', Object.keys(engine));

    // Create the highlighter core
    log('Creating highlighter core...');
    highlighterInstance = await (0, _core.createHighlighterCore)({
      langs: Object.values(LANGS),
      themes: [_githubDark.default],
      engine
    });
    log('Highlighter core created successfully');
    const loadedLangs = highlighterInstance.getLoadedLanguages();
    log('Loaded languages:', loadedLangs);
    return highlighterInstance;
  } catch (error) {
    log('Error creating highlighter:', error);
    throw new Error(`Failed to initialize syntax highlighter: ${error instanceof Error ? error.message : 'Unknown error'}`);
  }
}
function isValidLanguage(lang) {
  return lang in LANGS;
}
function getSafeLanguage(lang) {
  return isValidLanguage(lang) ? lang : 'plaintext';
}
//# sourceMappingURL=SyntaxContext.js.map