"use strict";

/* eslint-disable no-console */

import { createHighlighterCore } from '@shikijs/core';
import css from 'shiki/dist/langs/css.mjs';
import html from 'shiki/dist/langs/html.mjs';
import javascript from 'shiki/dist/langs/javascript.mjs';
import json from 'shiki/dist/langs/json.mjs';
import markdown from 'shiki/dist/langs/markdown.mjs';
import php from 'shiki/dist/langs/php.mjs';
import tsx from 'shiki/dist/langs/tsx.mjs';
import typescript from 'shiki/dist/langs/typescript.mjs';
import githubDark from 'shiki/dist/themes/github-dark.mjs';
const DEBUG = true;
function log(message, ...args) {
  if (DEBUG) {
    console.log(`[ShikiContext] ${message}`, ...args);
  }
}
class CustomRegexScanner {
  constructor(patterns) {
    this.regexes = patterns.map(pattern => {
      try {
        return new RegExp(pattern, 'g');
      } catch (e) {
        log('Error creating regex for pattern:', pattern, e);
        return /./g; // Fallback to match any character
      }
    });
  }
  findNextMatchSync(string, startPosition) {
    const stringContent = string.toString();
    for (let i = 0; i < this.regexes.length; i++) {
      const regex = this.regexes[i];
      if (!regex) continue;
      regex.lastIndex = startPosition;
      const match = regex.exec(stringContent);
      if (match) {
        return {
          index: match.index,
          captureIndices: [{
            start: match.index,
            end: match.index + match[0].length,
            length: match[0].length
          }],
          scanner: this
        };
      }
    }
    return null;
  }
  dispose() {
    this.regexes = [];
  }
}
class CustomRegexString {
  constructor(content) {
    this.content = content;
  }
  toString() {
    return this.content;
  }
}
function createCustomRegexEngine() {
  const engine = {
    createScanner(patterns) {
      return new CustomRegexScanner(patterns);
    },
    createString(content) {
      return new CustomRegexString(content);
    }
  };
  return engine;
}

// Process the language definitions
function processLanguage(lang) {
  if (!lang) {
    log('Warning: Null language definition');
    return null;
  }

  // Extract or create necessary properties
  const processed = {
    ...lang,
    id: lang.id || lang.scopeName?.split('.').pop() || 'unknown',
    scopeName: lang.scopeName || `source.${lang.id || 'unknown'}`,
    grammar: lang.grammar || {},
    embeddedLanguages: lang.embeddedLanguages || {}
  };
  log(`Processing language ${processed.id}:`, {
    scopeName: processed.scopeName,
    hasGrammar: Boolean(processed.grammar),
    embeddedLangs: Object.keys(processed.embeddedLanguages)
  });
  return processed;
}

// Map of all available languages with processed implementations
export const LANGS = {
  typescript: processLanguage(typescript),
  javascript: processLanguage(javascript),
  json: processLanguage(json),
  html: processLanguage(html),
  css: processLanguage(css),
  markdown: processLanguage(markdown),
  tsx: processLanguage(tsx),
  php: processLanguage(php),
  plaintext: {
    id: 'plaintext',
    scopeName: 'source.plaintext',
    grammar: {
      patterns: [{
        match: '.*',
        name: 'text.plain'
      }]
    }
  }
};
let highlighterInstance = null;
export async function getHighlighter() {
  if (highlighterInstance) {
    log('Returning cached highlighter instance');
    return highlighterInstance;
  }
  try {
    log('Creating new highlighter instance...');

    // Initialize custom regex engine
    log('Initializing custom regex engine...');
    const engine = createCustomRegexEngine();
    log('Engine created with capabilities:', Object.keys(engine));

    // Create the highlighter core
    log('Creating highlighter core...');
    highlighterInstance = await createHighlighterCore({
      langs: Object.values(LANGS),
      themes: [githubDark],
      engine
    });
    log('Highlighter core created successfully');
    const loadedLangs = highlighterInstance.getLoadedLanguages();
    log('Loaded languages:', loadedLangs);
    return highlighterInstance;
  } catch (error) {
    log('Error creating highlighter:', error);
    throw new Error(`Failed to initialize syntax highlighter: ${error instanceof Error ? error.message : 'Unknown error'}`);
  }
}
export function isValidLanguage(lang) {
  return lang in LANGS;
}
export function getSafeLanguage(lang) {
  return isValidLanguage(lang) ? lang : 'plaintext';
}
//# sourceMappingURL=SyntaxContext.js.map