import type { HighlighterCore, PatternScanner, RegexEngine, RegexEngineString } from '@shikijs/types'
/* eslint-disable no-console */
import type { IOnigMatch, OnigString } from '@shikijs/vscode-textmate'
import { createHighlighterCore } from '@shikijs/core'
import css from 'shiki/dist/langs/css.mjs'
import html from 'shiki/dist/langs/html.mjs'
import javascript from 'shiki/dist/langs/javascript.mjs'
import json from 'shiki/dist/langs/json.mjs'
import markdown from 'shiki/dist/langs/markdown.mjs'
import php from 'shiki/dist/langs/php.mjs'
import tsx from 'shiki/dist/langs/tsx.mjs'
import typescript from 'shiki/dist/langs/typescript.mjs'
import githubDark from 'shiki/dist/themes/github-dark.mjs'

const DEBUG = true
function log(message: string, ...args: any[]) {
  if (DEBUG) {
    console.log(`[ShikiContext] ${message}`, ...args)
  }
}

export type AvailableLanguage = 'php' | 'tsx' | 'typescript' | 'javascript' | 'json' | 'html' | 'css' | 'markdown' | 'plaintext'

class CustomRegexScanner implements PatternScanner {
  private regexes: RegExp[]

  constructor(patterns: string[]) {
    this.regexes = patterns.map((pattern) => {
      try {
        return new RegExp(pattern, 'g')
      }
      catch (e) {
        log('Error creating regex for pattern:', pattern, e)
        return /./g // Fallback to match any character
      }
    })
  }

  findNextMatchSync(string: string | OnigString, startPosition: number): IOnigMatch | null {
    const stringContent = string.toString()

    for (let i = 0; i < this.regexes.length; i++) {
      const regex = this.regexes[i]
      if (!regex)
        continue

      regex.lastIndex = startPosition
      const match = regex.exec(stringContent)

      if (match) {
        return {
          index: match.index,
          captureIndices: [
            {
              start: match.index,
              end: match.index + match[0].length,
              length: match[0].length,
            },
          ],
          scanner: this,
        } as IOnigMatch
      }
    }
    return null
  }

  dispose(): void {
    this.regexes = []
  }
}

class CustomRegexString implements RegexEngineString {
  content: string

  constructor(content: string) {
    this.content = content
  }

  toString(): string {
    return this.content
  }
}

function createCustomRegexEngine(): RegexEngine {
  const engine: RegexEngine = {
    createScanner(patterns: string[]): PatternScanner {
      return new CustomRegexScanner(patterns)
    },

    createString(content: string): RegexEngineString {
      return new CustomRegexString(content)
    },
  }

  return engine
}

// Process the language definitions
function processLanguage(lang: any): any {
  if (!lang) {
    log('Warning: Null language definition')
    return null
  }

  // Extract or create necessary properties
  const processed = {
    ...lang,
    id: lang.id || lang.scopeName?.split('.').pop() || 'unknown',
    scopeName: lang.scopeName || `source.${lang.id || 'unknown'}`,
    grammar: lang.grammar || {},
    embeddedLanguages: lang.embeddedLanguages || {},
  }

  log(`Processing language ${processed.id}:`, {
    scopeName: processed.scopeName,
    hasGrammar: Boolean(processed.grammar),
    embeddedLangs: Object.keys(processed.embeddedLanguages),
  })

  return processed
}

// Map of all available languages with processed implementations
export const LANGS: Record<AvailableLanguage, any> = {
  typescript: processLanguage(typescript),
  javascript: processLanguage(javascript),
  json: processLanguage(json),
  html: processLanguage(html),
  css: processLanguage(css),
  markdown: processLanguage(markdown),
  tsx: processLanguage(tsx),
  php: processLanguage(php),
  plaintext: {
    id: 'plaintext',
    scopeName: 'source.plaintext',
    grammar: {
      patterns: [{ match: '.*', name: 'text.plain' }],
    },
  },
}

let highlighterInstance: HighlighterCore | null = null

export async function getHighlighter(): Promise<HighlighterCore> {
  if (highlighterInstance) {
    log('Returning cached highlighter instance')
    return highlighterInstance
  }

  try {
    log('Creating new highlighter instance...')

    // Initialize custom regex engine
    log('Initializing custom regex engine...')
    const engine = createCustomRegexEngine()
    log('Engine created with capabilities:', Object.keys(engine))

    // Create the highlighter core
    log('Creating highlighter core...')
    highlighterInstance = await createHighlighterCore({
      langs: Object.values(LANGS),
      themes: [githubDark],
      engine,
    })

    log('Highlighter core created successfully')
    const loadedLangs = highlighterInstance.getLoadedLanguages()
    log('Loaded languages:', loadedLangs)

    return highlighterInstance
  }
  catch (error) {
    log('Error creating highlighter:', error)
    throw new Error(`Failed to initialize syntax highlighter: ${error instanceof Error ? error.message : 'Unknown error'}`)
  }
}

export function isValidLanguage(lang: string): lang is AvailableLanguage {
  return lang in LANGS
}

export function getSafeLanguage(lang: string): AvailableLanguage {
  return isValidLanguage(lang) ? lang : 'plaintext'
}
